import scala.io.Source
import scala.math._
import cl.asa.yaml._
import cl.asa.init._
import cl.asa.parse._
import cl.asa.result._
import cl.asa.yaml.frame._
import fileIO._
import scala.collection.JavaConversions._

/**
 * 正解データとASAの解析結果の比較
 */
object compare {
	print("起動中\r")
	private val Files = new YamlFile()
	private val Dicts = new LoadYaml(Files)
	private val Parser = new Parse(Dicts)
	private val Reader = new CsvReader()
	private var Scount = Map("all" -> 0, "true" -> 0, "false" -> 0, "NoVerb" -> 0)
	private var Rcount = Map("all" -> 0, "true" -> 0, "false" -> 0, "NoChunk" -> 0, "NoVerb" -> 0)

	def main(args: Array[String]): Unit = {
		var infile: String = ""

		args.foreach { arg =>
			arg match {
				case _ => infile = arg
			}
		}
		//infile = "/home1/ex/ikeda/data/wbccj20140823_correct.csv"
		infile = "/home1/ex/ikeda/data/vth20141008_correct.csv"
		val csvs = Source.fromFile(infile).getLines.map(Reader.parseLine(_).get).toSeq

		println("-start-")
		val start = System.currentTimeMillis()
		csvs.tail.foreach { csv =>
			val asa = Parser.parse(csv(14))
			compareResult(asa, csv)
		}
		val end = System.currentTimeMillis()
		println(end - start + "msec")
		this.OutputResult
		println("-done-")
	}

	def compareResult(asa: Result, csv: List[String]) {
		Scount = Scount.updated("all", Scount("all") + 1)
		val roles = csv.slice(1, 13).grouped(2).filter(_(1).nonEmpty).toSeq
		Rcount = Rcount.updated("all", Rcount("all") + roles.size)
		val verb = csv(0) //見出し語
		val semantic = csv(15) //語義
		val chunk = asa.chunks.filter(_.main.equals(verb)).applyOrElse(csv(13).toInt, (_: Int) => null)
		chunk.ne(null) match {
			case true =>
				this.compareSemantic(chunk, semantic)
				this.compareRole(chunk, roles)
			case false =>
				Scount = Scount.updated("NoVerb", Scount("NoVerb") + 1)
				Rcount = Rcount.updated("NoVerb", Rcount("NoVerb") + roles.size)
		}
	}

	def compareSemantic(chunk: Chunk, semantic: String) {
		semantic.equals(chunk.semantic) match {
			case true => Scount = Scount.updated("true", Scount("true") + 1)
			case false => Scount = Scount.updated("false", Scount("false") + 1)
		}
	}

	def compareRole(chunk: Chunk, roles: Seq[List[String]]) {
		roles.foreach { pair =>
			val role = pair(0)
			val term = pair(1)

			chunk.modifiedchunks.find { mod =>
				term.contains(mod.surface)
			} match {
				case Some(modchunk) =>
					role.equals(modchunk.semrole) match {
						case true => Rcount = Rcount.updated("true", Rcount("true") + 1)
						case false => Rcount = Rcount.updated("false", Rcount("false") + 1)
					}
				case None => Rcount = Rcount.updated("NoChunk", Rcount("NoChunk") + 1)

			}
		}
	}

	def OutputResult {
		println("全語義: " + Scount("all"))
		println(" 語義の一致: " + Scount("true"))
		println(" 語義の不一致: " + Scount("false"))
		println(" 取れなかった動詞: " + Scount("NoVerb"))
		println
		println("全意味役割: " + Rcount("all"))
		println(" 意味役割の一致: " + Rcount("true"))
		println(" 意味役割の不一致: " + Rcount("false"))
		println(" 取れなかったchunk: " + Rcount("NoChunk"))
		println(" 取れなかった動詞のchunk " + Rcount("NoVerb"))
	}

	def checkAsa(asa: Chunk, vth: List[String]) {
		println(vth(29) + ":" + asa.main)
		val semantic = vth.slice(30, 35).mkString("-")
		println("-ASA-")
		println(asa.semantic)
		for (mod <- asa.modifiedchunks) {
			println(mod.main + mod.part + ": " + mod.semrole + ": " + mod.score)
		}
		println("-vth-")
		println(semantic)
		val frames = Dicts.frames.getFrame(asa.main).get.frame
		val sem = frames.find(_.semantic.equals(semantic)).get
		val inst = sem.instance.find(_.cases.exists(t => vth(29).contains(t.noun))).get
		for (t <- inst.cases) {
			println(t.noun + t.part + ": " + t.semrole + ": " + t.weight)
		}

	}
}