import scala.io.Source
import scala.math._
import cl.asa.yaml._
import cl.asa.init._
import fileIO._

/**
 * シソーラスよりフレーム辞書の作成
 */
object createYamlDict {
	private val init = new YamlFile
	private val dicts = new LoadYaml(init)
	private val categorys = dicts.getCategorys

	def main(args: Array[String]): Unit = {
		var infile: String = "/home1/ex/ikeda/data/vths20141008.csv"
		var outfile: String = "test.yaml"
		args.foreach { arg =>
			arg match {
				case _ => infile = arg
			}
		}
		val csvs = Source.fromFile(infile)
		val dict = createDict(csvs.getLines)
		calculateWeight(dict)
		parseCaseConversion(dict)
		outputYaml(dict, outfile)
		println("done")
	}

	def createDict(lines: Iterator[String]): Dict = {
		val dict = new Dict
		lines.next
		for (line <- lines) {
			val vths = new CsvReader().parseLine(line).get
			val sem = vths.slice(30, 35).mkString("-")
			vths(36) match {
				case cmt if !cmt.contains("×") =>
					dict.frames.find(frame => frame.verb.equals(vths(2))) match {
						case Some(frame) =>
							frame.semantics.find(semantic => semantic.semantic.equals(sem)) match {
								case Some(semantic) =>
									semantic.instances = semantic.instances :+ createInstance(vths)
								case None =>
									frame.semantics = frame.semantics :+ createSemantic(vths)
							}
						case None =>
							dict.frames = dict.frames :+ createFrame(vths)
					}
				case _ => println(vths(0) + vths(2))
			}
		}
		return dict
	}

	def createFrame(vths: List[String]): Frame = {
		var frame = new Frame
		frame.verb = vths(2)
		frame.semantics = frame.semantics :+ createSemantic(vths)
		return frame
	}

	def createSemantic(vths: List[String]): Semantic = {
		var semantic = new Semantic
		semantic.semantic = vths.slice(30, 35).mkString("-")
		semantic.instances = semantic.instances :+ createInstance(vths)
		return semantic
	}

	def createInstance(vths: List[String]): Instance = {
		var instance = new Instance
		val voice = vths(39)
		val form = vths(40)
		for (n <- List(4, 9, 14, 19, 24)) {
			var cas = new Case
			cas.semrole = vths(n) //深層格
			cas.noun = vths(n + 2) //事例
			val pre_parts = vths(n + 1).split("[・?]").toList.distinct //表層格の候補
			cas.part = pre_parts(0)
			for (part <- pre_parts) {
				if (cas.noun.contains(part)) {
					cas.part = part //事例で使われている表層格
					cas.noun = vths(n + 2).replace(part, "") //事例内の名詞
				}
			}
			if (cas.noun.nonEmpty) cas.category = categorys.getCate(cas.noun) //事例内の名詞が所属するカテゴリー
			if (cas.semrole.nonEmpty || cas.noun.nonEmpty) instance.cases = instance.cases :+ cas
		}
		return instance
	}

	def calculateWeight(dict: Dict) {
		dict.frames.foreach { frame =>
			frame.semantics.foreach { semantic =>
				semantic.instances.foreach { instance =>
					instance.cases.foreach { cas =>
						val key = cas.category + cas.part
						val l_weight = calculateLocalWeight(semantic.instances, key)
						val g_weight = calculateGrobalWeight(frame.semantics, semantic.instances, key)
						cas.weight = l_weight * g_weight
					}
				}
			}
		}
	}
	/**
	 * 同語義事例間重みの計算
	 */
	def calculateLocalWeight(instances: List[Instance], key: String): Float = {
		val tf = 1
		val df = instances.count(instance => instance.cases.exists(cas => key.equals(cas.category + cas.part)))
		val idf = (log(instances.size / df) / log(2)) + 1
		val weight = 1 / (tf * idf)
		return weight.toFloat
	}

	/**
	 * 別語義事例間重みの計算
	 */
	def calculateGrobalWeight(semantics: List[Semantic], instances: List[Instance], key: String): Float = {
		val tf = instances.count(instance => instance.cases.exists(cas => key.equals(cas.category + cas.part)))
		val df = semantics.count(semantic => semantic.instances.exists(instance => instance.cases.exists(cas => key.equals(cas.category + cas.part))))
		val idf = (log(semantics.size / df) / log(2)) + 1
		val weight = tf * idf
		return weight.toFloat
	}

	def createPatternDic(lines: Iterator[String]): Map[String, Map[String, Map[String, String]]] = {
		var patterns = Map.empty[String, Map[String, Map[String, String]]]
		for (line <- lines) {
			val csv = new CsvReader().parseLine(line).get
			var pattern = Map.empty[String, Map[String, String]]
			if (csv(9).nonEmpty) {
				val elems = csv(9).split(';').apply(0).split(':')
				var causatives = Map.empty[String, String]
				elems.foreach { elem =>
					val t = elem.split("=>")
					causatives = causatives.updated(t(0), t(1))
				}
				pattern = pattern.updated("causative", causatives)
				val elems2 = csv(10).split(';').apply(0).split(':')
				var passives = Map.empty[String, String]
				elems2.foreach { elem =>
					val t = elem.split("=>")
					passives = passives.updated(t(0), t(1))
				}
				pattern = pattern.updated("passive", passives)
				patterns = patterns.updated(csv(0), pattern)
			}
		}
		return patterns
	}

	def parseCaseConversion(dict: Dict) {
		val senpats = Source.fromFile("/home1/ex/ikeda/data/pattern.csv")
		val patterns = createPatternDic(senpats.getLines)
		dict.frames.foreach { frame =>
			frame.semantics.foreach { semantic =>
				semantic.instances.foreach { instance =>
					val t = instance.cases.map(cas => cas.part)
					val parts: String = t.mkString("")
					patterns.get(parts) match {
						case Some(pattern) =>
							instance.cases.foreach { cas =>
								if (cas.part.nonEmpty) {
									cas.causative_part = pattern.apply("causative").apply(cas.part)
									cas.passive_part = pattern.apply("passive").apply(cas.part)
								}
							}
						case None =>
					}
				}
			}
		}
	}

	def outputYaml(dict: Dict, outfile: String) {
		val file = new OutputFiles("test.yaml")

		file.write("---")
		file.write("dict:")
		for (frame <- dict.frames) {
			file.write("- verb: " + frame.verb)
			file.write("  frame: ")
			for (semantic <- frame.semantics) {
				file.write("  - semantic: " + semantic.semantic)
				file.write("    instance: ")
				for (instance <- semantic.instances) {
					if (instance.cases.nonEmpty) file.write("    - cases: ")
					for (cas <- instance.cases) {
						file.write("      - part: " + cas.part)
						if (cas.causative_part.nonEmpty) file.write("        causative_part: " + cas.causative_part)
						if (cas.passive_part.nonEmpty) file.write("        passive_part: " + cas.passive_part)
						file.write("        noun: " + cas.noun)
						file.write("        category: " + cas.category)
						file.write("        semrole: " + cas.semrole)
						file.write("        weight: " + cas.weight)
						if (cas.voice.nonEmpty) file.write("        voice: " + cas.voice)
						if (cas.form.nonEmpty) file.write("        form: " + cas.form)
					}
				}
			}
		}
		file.close
	}

	class Dict {
		var frames = List.empty[Frame]
	}
	class Frame {
		var verb: String = ""
		var semantics = List.empty[Semantic]
	}

	class Semantic {
		var semantic: String = ""
		var instances = List.empty[Instance]
	}
	class Instance {
		var cases = List.empty[Case]
	}
	class Case {
		var part: String = ""
		var causative_part = ""
		var passive_part = ""
		var noun: String = ""
		var category: String = ""
		var semrole: String = ""
		var voice: String = ""
		var form: String = ""
		var weight: Float = 0F
	}
}