package cl.asa.parse.semantic
/**
 * 語義，意味役割を付与するためのクラス
 */
import cl.asa.result._
import cl.asa.yaml.frame._
import cl.asa.yaml.category._
import cl.asa.yaml.verb._
import cl.asa.yaml._

class Sematter(verbs: Verbs, frames: frame.Dict, categorys: category.Dict) {
	private val calc = new Calculate(frames)
	private val adjunct = new Adjunct()

	def parse(result: Result): Result = {
		val verbchunks: Seq[Chunk] = getSemChunks(result)
		verbchunks.foreach { verbchunk =>
			val linkchunks = getLinkChunks(verbchunk)
			setAnotherPart(linkchunks)
			calc.getFrame(verbchunk.main, linkchunks) match {
				case Some((semantic, similar, insts)) =>
					setSemantic(semantic, similar, verbchunk)
					setSemRole(insts)
					adjunct.parse(verbchunk.modifiedchunks)
				case None =>
			}
		}
		return result
	}

	/**
	 * 助詞の言い換え候補があるものに対して，言い換えの助詞を付与
	 */
	private def setAnotherPart(chunks: Seq[Chunk]) {
		chunks.foreach { chunk =>
			chunk.morphs.foreach { morph =>
				morph.pos match {
					case pos if pos.contains("格助詞") && Seq("に", "へ").contains(chunk.part) =>
						chunk.another_parts = Seq("に", "へ").diff(chunk.part)
					case pos if pos.contains("係助詞") =>
						chunk.another_parts = Seq("が", "を")
					case _ => null
				}
			}
		}
	}

	/**
	 * 係り先である節を取得
	 */
	def getSemChunks(result: Result): Seq[Chunk] = {
		val chunks: Seq[Chunk] = result.chunks.filter { chunk =>
			!chunk.ctype.equals("elem") && frames.isFrame(chunk.main)
		}
		return chunks
	}
	/**
	 * 係り先の節を渡して，その係り元を取得
	 */
	private def getLinkChunks(verbchunk: Chunk): Seq[Chunk] = {
		val linkchunks = verbchunk.modifiedchunks.filter(_.ctype.equals("elem"))
		return linkchunks
	}

	/**
	 * 曖昧性を解消したフレームのデータより語義を付与
	 * @param semantic Calcurateクラスより取得したデータ
	 * @param verbchunk 語義を付与する文節
	 */
	private def setSemantic(semantic: String, similar: Float, verbchunk: Chunk) {
		verbchunk.semantic = semantic
		verbchunk.similar = similar
	}

	/**
	 * 曖昧性を解消したフレームのデータより意味役割を付与
	 * @param semantic Calcurateクラスより取得したデータ
	 */
	private def setSemRole(insts: Seq[(Float, Case, Chunk)]) {
		insts.foreach { instset =>
			val (similar, icase, chunk) = instset
			chunk.semrole = icase.semrole
			chunk.similar = similar

		}
	}
}
