package cl.asa.yaml
/**
 * yamlファイルの各辞書を読み込むクラス
 */
import org.yaml.snakeyaml._
import java.io._
import cl.asa.yaml.frame._
import cl.asa.yaml.category._
import cl.asa.yaml.cchart._
import cl.asa.yaml.verb._
import cl.asa.init._

class LoadYaml(files: YamlFile) {
	val yaml = new Yaml()
	val frames: frame.Dict = loadFrames(files.getFrame)
	val categorys: category.Dict = loadCategorys(files.getCategory)
	val verbs: Verbs = loadVerbs(files.getVerb)
	val ccharts: cchart.Dict = loadCcharts(files.getCchart)
	val idioms: idiom.Dict = loadIdioms(files.getIdiom)
	val filters: filter.Dict = loadFilters(files.getFilter)

	def getIdioms(): idiom.Dict = {
		return idioms
	}
	def getCategorys(): category.Dict = {
		return categorys
	}

	def getCcharts(): cchart.Dict = {
		return ccharts
	}

	def getVerbs(): Verbs = {
		return verbs
	}

	def getFrames(): frame.Dict = {
		return frames
	}
	def getFilters(): filter.Dict = {
		return filters
	}

	private def loadIdioms(file: String): idiom.Dict = {
		val idioms = yaml.loadAs(new FileReader(new File(file)),classOf[idiom.Dict])
		//val idioms = yaml.loadAs(getClass().getClassLoader().getResourceAsStream(file),classOf[idiom.Dict])
		return idioms
	}

	private def loadCcharts(file: String): cchart.Dict = {
		val ccharts = yaml.loadAs(new FileReader(new File(file)),classOf[cchart.Dict])
		//val ccharts = yaml.loadAs(getClass().getClassLoader().getResourceAsStream(file),classOf[cchart.Dict])
		return ccharts
	}

	private def loadVerbs(file: String): Verbs = {
		val verbs = yaml.loadAs(new FileReader(new File(file)),classOf[Verbs])
		//val verbs = yaml.loadAs(getClass().getClassLoader().getResourceAsStream(file),classOf[Verbs])
		return verbs
	}

	private def loadCategorys(file: String): category.Dict = {
		val categorys = yaml.loadAs(new FileReader(new File(file)),classOf[category.Dict])
		//val categorys = yaml.loadAs(getClass().getClassLoader().getResourceAsStream(file),classOf[category.Dict])
		return categorys
	}

	private def loadFrames(file: String): frame.Dict = {
		val frames = yaml.loadAs(new FileReader(new File(file)),classOf[frame.Dict])
		//val frames = yaml.loadAs(getClass().getClassLoader().getResourceAsStream(file),classOf[frame.Dict])
		return frames
	}
	
	private def loadFilters(file: String): filter.Dict = {
		val filters = yaml.loadAs(new FileReader(new File(file)),classOf[filter.Dict])
		//val filters = yaml.loadAs(getClass().getClassLoader().getResourceAsStream(file),classOf[filter.Dict])
		return filters
	}
}